#!/usr/bin/env python

##############################################################################
#
# Copyright (C) Zenoss, Inc. 2015, all rights reserved.
#
# This content is made available according to terms specified in
# License.zenoss under the directory where your Zenoss product is installed.
#
##############################################################################

import logging
import os
import site
import unittest
import Globals
from Products.ZenUtils.Utils import unused

unused(Globals)

site.addsitedir(os.path.join(os.path.dirname(os.path.realpath(__file__)), ".."))
data_dir = os.path.join(os.path.dirname(os.path.realpath(__file__)), "data")

from ZenPacks.zenoss.ZenPackLib import zenpacklib
import yaml

logging.basicConfig(level=logging.INFO)
LOG = logging.getLogger('zen.zenpacklib.tests')


def load_cfg_dict(filename, zenpack_name):
    g = dict(zenpacklib=zenpacklib)
    l = dict(CFG={})
    py_filename = os.path.join(data_dir, filename)
    LOG.info("Loading %s zenpack spec from %s" % (zenpack_name, py_filename))
    execfile(py_filename, g, l)
    CFG = l.get('CFG')
    CFG['name'] = zenpack_name
    return CFG


def dummy_zenpack_path(zenpack_name):
    return "/tmp"

zenpacklib.get_zenpack_path = dummy_zenpack_path


class TestYAML(unittest.TestCase):
    disableLogging = False
    maxDiff = None

    def test_small_zenpack(self):
        self._yamltest('small.py', 'SmallZenPack')

    def test_openstack1(self):
        self._yamltest('openstack1.py', 'OpenStackInfrastructure')

    def test_hp_proliant1(self):
        self._yamltest('hp_proliant1.py', 'HPProliant')

    def _yamltest(self, filename, zenpackname):
        # Load the legacy (python dictionary) zenpack parameter
        LOG.info("Reading %s data from %s" % (zenpackname, filename))
        cfg_dict = load_cfg_dict(filename, zenpackname)

        # Convert to ZenPackSpecParams form.  This validates the input
        # dict, as well as making it possible to dump it to YAML form.
        LOG.info("Constructing ZenPackSpecParams")
        specparams = zenpacklib.ZenPackSpecParams(**cfg_dict)

        # Now convert to yaml.
        LOG.info("Exporting from ZenPackSpecParams to YAML")
        exported_yaml = yaml.dump(specparams, Dumper=zenpacklib.Dumper)
        LOG.debug(exported_yaml)

        # now re-load this..
        LOG.info("Reloading the generated YAML from ZenPackSpecParams")
        reloaded_spec = yaml.load(exported_yaml, Loader=zenpacklib.Loader)

        # We now want to confirm that the reloaded spec is equivalent to the orginal
        # spec we exported.
        #
        # However, in order to export it, we instantiated it as a ZenPackSpecParams object.
        # When we loaded it, we built a ZenPackSpec object.  These can not
        # currently be directly compared (__eq__ in Spec could allow
        # for this, but ZenPackSpec includes a lot of initializations with default
        # values that ZenPackSpecParams does not try to do, so in practice,
        # they won't match exactly).
        #
        # In order to do the comparison correctly, we will just construct a
        # regular ZenPackSpec from the original cfg_dict and use that.
        LOG.info("Constructing ZenPackSpec")
        original_spec = zenpacklib.ZenPackSpec(**cfg_dict)

        # If they don't match, the problem could be in several places, then-
        # in the building of the original ZenPackSpecParams, in its exporting,
        # in the building of the ZenPackSpec, etc.  So let's hope they match.
        self.assertEqual(
            original_spec, reloaded_spec,
            "Compare original params from %s with result of export to and reimport from YAML (ZenPackSpec)" % filename)

        # OK, now let's repeat the steps, but this time export from the specs,
        # rather than the specparams.  Should be identical, since both support
        # yaml export, but let's make sure.

        LOG.info("Exporting from ZenPackSpec to YAML")
        exported_yaml = yaml.dump(original_spec, Dumper=zenpacklib.Dumper)
        LOG.debug(exported_yaml)

        # now re-load this..
        LOG.info("Reloading the generated YAML from ZenPackSpec")
        reloaded_spec = yaml.load(exported_yaml, Loader=zenpacklib.Loader)

        # TODO:
        # The order attribute for classes and properties are forced into a
        # specific range, and when we retrieve it for YAML export, we get
        # that modified value, not the original one that was passed in.
        #
        # Therefore, this test will fail unless we force the values back to
        # what we expected them to be.   The proper fix will be for
        # the original passed-in order to be exported to yaml, and a derived
        # order (with the range normalized) to be used internally.  Once that
        # fix is in place, this workaround can be removed.
        for class_name in reloaded_spec.classes:
            if class_name in original_spec.classes:
                original_class = original_spec.classes[class_name]
                reloaded_class = reloaded_spec.classes[class_name]
                reloaded_class.order = original_class.order
                for property_name in reloaded_class.properties:
                    if property_name in original_class.properties:
                        original_property = original_class.properties[property_name]
                        reloaded_property = reloaded_class.properties[property_name]
                        reloaded_property.order = original_property.order

        # and re-compare
        self.assertEqual(
            original_spec, reloaded_spec,
            "Compare original params from %s with result of export to and reimport from YAML (ZenPackSpec)" % filename)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestYAML))
    return suite


if __name__ == "__main__":
    from zope.testrunner.runner import Runner
    runner = Runner(found_suites=[test_suite()])
    runner.run()
